/*
 * dbf.c - Database formats, the blocks ordered by DBW.
 */
#include "fuzzy.h"
#include "dbf.h"
#include "register.h"

/*
 * This module will contain the functions necessary to manipulate
 * the entities ordered by the DBW module.  While the DBW will do
 * lookups, removals, additions, etc. to the various DBF blocks,
 * it will not actually do the changes.  This module will contain
 * all room, mobile, object, zone, shop operations to be executed
 * in the world.
 *
 * This also has the unique situation of having a single format
 * (SQL) handle both the DBF and DBW sides of the manipulations.
 * So while a DBW module may SELECT for rooms, it will be called
 * again in the DBF section to actually read/write them.  Locking
 * should be interesting.
 *
 * -gg 6/10/99
 *
 * This was known as the 'database_format' module in my previous
 * work. Just to help me keep the TLAs in check.
 *
 * -gg 7/11/99
 */

extern struct data_format_mobile_operations dbf_mobile;
extern struct data_format_object_operations dbf_object;
extern struct data_format_room_operations dbf_room;
extern struct data_format_shop_operations dbf_shop;
extern struct data_format_zone_operations dbf_zone;

struct data_format_operations dbf_ops = {
  &dbf_mobile,
  &dbf_object,
  &dbf_room,
  &dbf_shop,
  &dbf_zone
};

int dbf_init(void)
{
  /* Nothing... */
  return TRUE;
}

void *dbf_lookup(const char *name)
{
  const char *names[] = { "mobile", "object", "room", "shop", "zone" };
  int i;

  for (i = 0; i < 5; i++)
    if (!g_strcasecmp(name, names[i]))
      return GINT_TO_POINTER(i);

  return GINT_TO_POINTER(-1);
}
