#include "fuzzy.h"
#include "dio.h"
#include "dbd.h"
#include "dfp.h"
#include "dbw.h"
#include "dbf.h"
#include "index.h"

#include <unistd.h> /* chdir */
void bootstrap(void); /* HACK */

int main(int argc, char *argv[])
{
  /* Data I/O */
  dio_init();

  /* Low-level SQL drivers for dio. */
  dbd_init();

  /* Parsers of area format on disk. */
  dfp_init();

  /* Organization of world units in memory. */
  dbw_init();

  /* Lowest level blocks manipulated in dbw. */
  dbf_init();

  bootstrap();

  return TRUE;
}

World *db_create(const char *name, struct database_world_operations *dbw, struct database_format_operations *dbf)
{
  int i;
  World *world = g_new0(World, 1);

  for (i = 0; i < 5; i++) {
    world->dbw[i] = dbw;
    world->data[i] = dbw->create(name);
    world->dbf[i] = dbf;
  }
  world->name = g_strdup(name);
  return world;
}

/* HACK */
void bootstrap(void)
{
  World *database;
  struct data_io_operations *f_ops;
  struct database_world_operations *dbw_ops;
  struct database_format_operations *dbf_ops;
  void *handle;
  char *data, *method, *filename;

  if (!(f_ops = dio_lookup("mmap syscall")))
    dlog("dio_lookup fail");
  if (!(handle = f_ops->open("../dbp2.config")))
    dlog("->open failed");
  if (!(data = f_ops->data(handle)))
    dlog("->read failed: %p", handle);

  *strchr(data, '\n') = '\0';

  method = strchr(data, ' ');
  *method++ = '\0';

  filename = strrchr(data, '/');
  *filename++ = '\0';

  if (chdir(data) < 0)
    dlog("chdir failed: '%s'", data);
  if (!(dbw_ops = dbw_lookup(method)))
    dlog("dbw_lookup failed: '%s'", method);
  if (!(dbf_ops = dbf_lookup(NULL)))
    dlog("dbf_lookup failed: NULL");
  if (!(database = db_create("main world", dbw_ops, dbf_ops)))
    dlog("->create failed");

  index_load(database, filename);

  f_ops->close(handle);
}

