/*
 * dbd.c - Database driver functions.  Designed to be lower level than dio_sql.
 */
#include "fuzzy.h"
#include "dio.h"
#include "dbd.h"
#include "register.h"

/*
 * It would be quite cool to embed Perl or Python code to take
 * advantage of their database interfaces and drivers.  Then
 * ODBC and other drivers come for free.  Since Perl will most
 * likely be a supported scripting language, we'll have access
 * to it anyway. -gg 8/7/99
 */

static HandlerList *dbd_index;

int dbd_init(void)
{
  dbd_index = handler_list_create();

  /* Insert new database drivers below. */

  DBD_REGISTER(mysql);

  /* New database drivers go above. */
  return TRUE;
}

/*
 * Maps database name to database driver functions.
 */
struct database_driver_operations *dbd_lookup(const char *type)
{
  return handler_lookup(dbd_index, type);
}

struct dbd_internal {
  struct database_driver_operations *dbd;
  char *query;
  void *handle;
};

DBD *dbd_open(const char *name, char *path)
{
  struct dbd_internal *data, first;
  struct database_driver_operations *dbd_ops;

  if (!(dbd_ops = dbd_lookup(name)))
    return NULL;

  first.query = g_strdup(sread(&path, "\n\n"));
  first.dbd = dbd_ops;
  first.handle = dbd_ops->open(path);

  if (!first.handle) {
    g_free(first.query);
    return NULL;
  }

  data = g_new0(struct dbd_internal, 1);
  *data = first;
  return data;
}

char *dbd_data(DBD *handle)
{
  struct dbd_internal *idata = (struct dbd_internal *)handle;

  return idata->dbd->query(idata->handle, idata->query);
}

void dbd_close(DBD *handle)
{
  struct dbd_internal *idata = (struct dbd_internal *)handle;

  idata->dbd->close(idata->handle);
  g_free(idata->query);
  g_free(idata);
}
