typedef void DBD;

struct database_driver_operations {
  const char *name;
  DBD *(*open)(char *path);
  char *(*query)(DBD *driver, const char *query);
  int (*execute)(DBD *driver, const char *query);
  void (*close)(DBD *);
};

int dbd_init(void);
struct database_driver_operations *dbd_lookup(const char *type);
DBD *dbd_open(const char *name, char *path);
char *dbd_data(DBD *handle);
void dbd_close(DBD *handle);

#define DBD_DEFINE(type)	\
	extern struct database_driver_operations dbd_##type

#define DBD_DECLARE(type, name)	\
	DBD *dbd_##type##_open(char *path);		\
	char *dbd_##type##_query(DBD *dbd, const char *query);	\
	int dbd_##type##_execute(DBD *dbd, const char *query);	\
	void dbd_##type##_close(DBD *driver);			\
								\
	struct database_driver_operations dbd_##type = {	\
		name,			\
		dbd_##type##_open,	\
		dbd_##type##_query,	\
		dbd_##type##_execute,	\
		dbd_##type##_close	\
	}

#define DBD_REGISTER(type)	\
	do {	\
	extern struct database_driver_operations dbd_##type;	\
	handler_register(dbd_index, &dbd_##type);	\
	} while(0)
