#include <mysql/mysql.h>
#include "fuzzy.h"
#include "dbd.h"
#include "options.h"

DBD_DECLARE(mysql, "MySQL");

struct options_list dbd_mysql_options[] = {
  { "host",       NULL },
  { "username",   NULL },
  { "password",   NULL },
  { "database",   NULL },
  { "port",       NULL },
  { "unixsocket", NULL },
  { NULL,         NULL }
};

DBD *dbd_mysql_open(char *path)
{
  MYSQL *mysql, *dbh;
  int client_flags = 0, port = 0;
  char *host, *user, *password, *database, *unixsock;

  mysql = mysql_init(NULL);
  parse_options(path, dbd_mysql_options);

  host = dbd_mysql_options[0].value;
  user = dbd_mysql_options[1].value;
  password = dbd_mysql_options[2].value;
  database = dbd_mysql_options[3].value;
  port = dbd_mysql_options[4].value ? atoi(dbd_mysql_options[4].value) : 0;
  unixsock = dbd_mysql_options[5].value;

  dlog("Host[%s] User[%s] Password[%s]", host ? host : "*", user ? user : "*", password ? password : "*");
  dlog("Database[%s] Port[%d] UnixSock[%s] Flags[%d]", database ? database : "*", port, unixsock ? unixsock : "*", client_flags);

  if (!(dbh = mysql_real_connect(mysql, host, user, password, database, port, unixsock, client_flags))) {
    mlog(0, 0, "Connection failed: %s", mysql_error(mysql));
    mysql_close(mysql);
  }

  free_options(dbd_mysql_options);

  return dbh;
}

char *dbd_mysql_query(DBD *driver, const char *query)
{
  MYSQL *dbh = (MYSQL *)driver;
  unsigned int num_fields, i, retlen;
  MYSQL_RES *result;
  MYSQL_ROW row;
  char *retval;

  if (mysql_query(dbh, query)) {
    mlog(0, 0, "Query failed [%s]: %s", query, mysql_error(dbh));
    return NULL;
  }

  if (!(result = mysql_store_result(dbh))) {
    if (mysql_num_fields(dbh) == 0)
      mlog(0, 0, "Query was not a select. [%s]", query);
    else
      mlog(0, 0, "Error handling request: [%s]", query);
    return NULL;
  }

  if (mysql_num_rows(result) == 0)
    return NULL;

  num_fields = mysql_num_fields(result);

  retval = g_new0(char, 1);
  retlen = 1;

  while ((row = mysql_fetch_row(result)))
    for (i = 0; i < num_fields; i++) {
      retlen += strlen(row[i]);
      retval = g_renew(char, retval, retlen);
      strcat(retval, row[i]);
    }

  mysql_free_result(result);
  return retval;
}

int dbd_mysql_execute(DBD *driver, const char *query)
{
  MYSQL *dbh = (MYSQL *)driver;

  if (mysql_query(dbh, query)) {
    mlog(0, 0, "Execute [%s] failed: %s", query, mysql_error(dbh));
    return -1;
  }

  return mysql_affected_rows(dbh);
}

void dbd_mysql_close(DBD *driver)
{
  mysql_close(driver);
}
