/*
 * dbw.c - Database <-> World translator.
 *	Run-time side.
 */
#include "fuzzy.h"
#include "dbw.h"
#include "register.h"

/*
 * MODULE DESCRIPTON
 *
 * This module will bring order unto the DBF blocks of information.
 * None of the changes will be done by this section, it will simply
 * do lookups, removals, additions, etc.  While this may sound
 * limited, it is actually a relatively complicated portion of the
 * code.  Removing rooms from arrays, linked lists, hash tables,
 * trees, and more, all while preserving a generic interface.  It
 * shall be worth it.
 *
 * Going into absurdity, it is possible to make an FTP DBW module.
 * A lookup would login and download a file.  The deletion and
 * addition would be obvious.  The method of updating with DFP
 * would probably also need a special FTP DFP module, but it will
 * be workable in theory.  (DFP/DBW are separated because many
 * organizational methods operate on pointers and thus can share
 * the same in-core backing, such as a struct.)
 *
 * Other absurd methods, such as a binary file for every entity
 * spread out on an NFS mounted filesystem that is used by many
 * MUDs simultaneously, are possible.  Not that I'd recommend
 * this as a normal thing.
 *
 * -gg 6/10/99
 */

/*
 * Yup, definitely need a 'register.c'.  That'll happen when
 * the below code is Glib-ified but it's not a priority.
 *
 * -gg 7/11/99
 */

static HandlerList *dbw_index;

int dbw_init(void)
{
  dbw_index = handler_list_create();

  /* Insert new database handlers below. */

  DBW_REGISTER(hash);
  DBW_REGISTER(tree);
  DBW_REGISTER(sql);

  /* New database handlers go above. */
  return TRUE;
}

void *dbw_lookup(const char *name)
{
  return handler_lookup(dbw_index, name);
}
