/*
 * Database World: GLib Hash Table implementation.
 */
#include "fuzzy.h"
#include "dbw.h"

DBW_DECLARE(hash, "GLib hash table");

db_t *hash_create(const char *name)
{
  return g_hash_table_new(g_direct_hash, g_direct_equal);
}

void hash_destroy(db_t *world)
{
  g_hash_table_foreach(world, NULL /* callback goes here, DBF func */, NULL);
  g_hash_table_destroy(world);
}

int hash_add(db_t *world, int key, void *value)
{
  if (g_hash_table_lookup(world, GINT_TO_POINTER(key)))
    return FALSE;

  g_hash_table_insert(world, GINT_TO_POINTER(key), value);
  return TRUE;
}

int hash_remove(db_t *world, int key)
{
  if (!g_hash_table_lookup(world, GINT_TO_POINTER(key)))
    return FALSE;

  g_hash_table_remove(world, GINT_TO_POINTER(key));
  return TRUE;
}
  
void *hash_lookup(db_t *world, int key)
{
  return g_hash_table_lookup(world, GINT_TO_POINTER(key));
}

int hash_size(db_t *world)
{
  return g_hash_table_size(world);
}
