/*
 * Database World: GLib Hash Table implementation.
 */
#include "fuzzy.h"
#include "dbw.h"
#include "dbd.h"

DBW_DECLARE(sql, "Generic SQL");

struct dbw_sql_data {
  struct database_driver_operations *dbd;
  DBD *dbh;
};

db_t *sql_create(const char *name)
{
  struct database_driver_operations *database;
  DBD *dbh;
  struct dbw_sql_data *dsd;

  if ((database = dbd_lookup("MySQL")) == NULL)
    return NULL;

  if ((dbh = database->open("program:user:password:database")) == NULL)
    return NULL;

  dsd = g_new0(struct dbw_sql_data, 1);
  dsd->dbd = database;
  dsd->dbh = dbh;
  return dsd;
}

void sql_destroy(db_t *world)
{
  struct dbw_sql_data *dsd = (struct dbw_sql_data *)world;

  dsd->dbd->close(dsd->dbh);
  g_free(dsd);
}

int sql_add(db_t *world, int key, void *value)
{
  struct dbw_sql_data *dsd = (struct dbw_sql_data *)world;

  return dsd->dbd->execute(dsd->dbh, "INSERT INTO foo VALUES('bar')");
}

int sql_remove(db_t *world, int key)
{
  struct dbw_sql_data *dsd = (struct dbw_sql_data *)world;

  return dsd->dbd->execute(dsd->dbh, "DELETE FROM foo WHERE bar='baz';");
}
  
void *sql_lookup(db_t *world, int key)
{
  struct dbw_sql_data *dsd = (struct dbw_sql_data *)world;

  return dsd->dbd->query(dsd->dbh, "SELECT FROM foo WHERE bar='baz')");
}

int sql_size(db_t *world)
{
  struct dbw_sql_data *dsd = (struct dbw_sql_data *)world;

  return atoi(dsd->dbd->query(dsd->dbh, "SELECT COUNT(*) FROM foo"));
}
