/*
 * Database World: GLib Hash Table implementation.
 */
#include "fuzzy.h"
#include "dbw.h"

DBW_DECLARE(tree, "GLib balanced binary tree");

int tree_compare(const void *a, const void *b)
{
  return (long)a - (long)b;
}

int tree_search(void *a, void *b)
{
  return (long)a - (long)b;
}

db_t *tree_create(const char *name)
{
  return g_tree_new(tree_compare);
}

void tree_destroy(db_t *world)
{
  //g_tree_traverse();
  g_tree_destroy(world);
}

int tree_add(db_t *world, int key, void *value)
{
  if (g_tree_search(world, tree_search, GINT_TO_POINTER(key)))
    return FALSE;

  g_tree_insert(world, GINT_TO_POINTER(key), value);
  return TRUE;
}

int tree_remove(db_t *world, int key)
{
  if (!g_tree_search(world, tree_search, GINT_TO_POINTER(key)))
    return FALSE;

  g_tree_remove(world, GINT_TO_POINTER(key));
  return TRUE;
}
  
void *tree_lookup(db_t *world, int key)
{
  return g_tree_search(world, tree_search, GINT_TO_POINTER(key));
}

int tree_size(db_t *world)
{
  return g_tree_nnodes(world);
}
