typedef int (*parse_func_t)(World *world, char **data);

struct data_parser_operations {
  const char *id;
  union {
    struct {
      int (*mobile)(World *world, char **data);
      int (*object)(World *world, char **data);
      int (*room)(World *world, char **data);
      int (*shop)(World *world, char **data);
      int (*zone)(World *world, char **data);
    } type;
    parse_func_t num[5];
  } parse_by;
};

int dfp_init(void);
void *dfp_lookup(const char *name);
int dfp_parse(World *world, struct data_parser_operations *dfp, int type, char **text);

#define DFP_DECLARE(type, name)	\
	int type##_parse_room(World *world, char **data);	\
	int type##_parse_shop(World *world, char **data);	\
	int type##_parse_object(World *world, char **data);	\
	int type##_parse_mobile(World *world, char **data);	\
	int type##_parse_zone(World *world, char **data);	\
								\
        struct data_parser_operations dfp_##type = {	\
	  name, { { type##_parse_mobile, type##_parse_object,	\
	  type##_parse_room, type##_parse_shop,	\
	  type##_parse_zone } }				\
	}

#define DFP_REGISTER(type)	\
	do {	\
	extern struct data_parser_operations dfp_##type;	\
	handler_register(dfp_index, &dfp_##type);		\
	} while (0)
