/*
 * dfp_circle.c - Data Format Parser, CircleMUD
 */
#include "fuzzy.h"
#include "dfp.h"
#include "dio.h"

DFP_DECLARE(circle, "CircleMUD");

static GScannerConfig parser_template = {
  (
   " \t\r\n"
   )			/* cset_skip_characters */,
  (
   G_CSET_a_2_z
   "_"
   G_CSET_A_2_Z
   )			/* cset_identifier_first */,
  (
   G_CSET_a_2_z
   "_0123456789"
   G_CSET_A_2_Z
   G_CSET_LATINS
   G_CSET_LATINC
   )			/* cset_identifier_nth */,
  ( "*\n" )		/* cpair_comment_single */,
  
  FALSE			/* case_sensitive */,
  
  TRUE			/* skip_comment_multi */,
  TRUE			/* skip_comment_single */,
  TRUE			/* scan_comment_multi */,
  TRUE			/* scan_identifier */,
  FALSE			/* scan_identifier_1char */,
  FALSE			/* scan_identifier_NULL */,
  TRUE			/* scan_symbols */,
  FALSE			/* scan_binary */,
  TRUE			/* scan_octal */,
  TRUE			/* scan_float */,
  TRUE			/* scan_hex */,
  FALSE			/* scan_hex_dollar */,
  TRUE			/* scan_string_sq */,
  TRUE			/* scan_string_dq */,
  TRUE			/* numbers_2_int */,
  FALSE			/* int_2_float */,
  FALSE			/* identifier_2_string */,
  TRUE			/* char_2_token */,
  FALSE			/* symbol_2_token */,
  FALSE			/* scope_0_fallback */,
};

enum {
  DFP_CIRCLE_HASH	= G_TOKEN_LAST
} dfp_circle_tokens;

static const struct {
  char *name;
  unsigned int value;
} symbols[] = {
  { "#", DFP_CIRCLE_HASH }
};

static const unsigned int num_symbols = sizeof(symbols) / sizeof(symbols[0]);

int circle_parse_room(World *world, char **data)
{
  GScanner *scan;
  int count;
  int token;
  Room *room;

  /* Do a bit of setup. */
  scan = g_scanner_new(&parser_template);
  g_scanner_input_text(scan, *data, strlen(*data));

  /* Auto-converted string table? */
  g_scanner_freeze_symbol_table(scan);
  for (count = 0; count < num_symbols; count++)
    g_scanner_add_symbol(scan, symbols[count].name, GINT_TO_POINTER(symbols[count].value));
  g_scanner_thaw_symbol_table(scan);

  while (g_scanner_peek_next_token(scan) != G_TOKEN_EOF) {
    dlog("Line: %d", g_scanner_cur_line(scan));

    token = g_scanner_get_next_token(scan);
    dlog("Token: %d", scan->token);

    if (token == DFP_CIRCLE_HASH) {
#if 0
      if (g_scanner_get_next_token(scan) != G_TOKEN_INT)
        dlog("oops, # without number");
      if (!(room = world->dbf[DBF_ROOM]->create()))
        dlog("dbf->create failed");
      if (!world->dbw[DBF_ROOM]->add(world->data[DBF_ROOM], scan->value.v_int, room)
        dlog("dbw->add failed");
#endif
    }
  }

  g_scanner_destroy(scan);
  return -1;
}

int circle_parse_mobile(World *world, char **data)
{
  return -1;
}

int circle_parse_object(World *world, char **data)
{
  return -1;
}

int circle_parse_shop(World *world, char **data)
{
  return -1;
}

int circle_parse_zone(World *world, char **data)
{
  return -1;
}

