/*
 * Data IO.
 */
#include "fuzzy.h"
#include "dio.h"
#include "register.h"

/* Private */
static HandlerList *dio_index;

int dio_init(void)
{
  dio_index = handler_list_create();

  /* Insert new data handlers below. */

  DIO_REGISTER(mmap);
  DIO_REGISTER(fread);
  DIO_REGISTER(read);
  DIO_REGISTER(sql);

  /* New data handlers go above. */
  return TRUE;
}

void *dio_lookup(const char *name)
{
  return handler_lookup(dio_index, name);
}

/* ----- */

struct dio_internal {
  struct data_io_operations *dio;
  void *handle;
};

DIO *dio_open(const char *name, char *path)
{
  struct dio_internal *data, first;

  if (!(first.dio = dio_lookup(name)))
    return NULL;

  if (!(first.handle = first.dio->open(path)))
    return NULL;

  data = g_new0(struct dio_internal, 1);
  *data = first;
  return data;
}

char *dio_data(DIO *handle)
{
  struct dio_internal *internal = (struct dio_internal *)handle;

  if (!internal)
    return NULL;

  return internal->dio->data(internal->handle);
}

void dio_close(DIO *handle)
{
  struct dio_internal *internal = (struct dio_internal *)handle;

  if (!internal)
    return;

  internal->dio->close(internal->handle);
  g_free(handle);
}

/* ------------------------------------------------------------------------------------------------------------------------------ */

char *sread(char **data, char *terminator)
{
  char *target, *orig;
  int len;

  g_return_val_if(!data || !*data || !**data, NULL);

  len = strlen(terminator);
  orig = *data;
  target = strstr(orig, terminator);
  if (!target)
    target = sread(data, "\0");

  memset(target, 0, len);
  *data = target + len;
  return orig;
}

char *sread_line(char **data)
{
  return sread(data, "\n");
}

char *sread_all(char **data)
{
  return sread(data, "\0");
}

char *sread_string(char **data)
{
  char *ret = sread(data, "~");
  if (!ret)
    return NULL;
  sread(data, "\n");
  return strdup(ret);
}
