/*
 * fread() file io.
 */
#include <stdio.h>
#include "fuzzy.h"
#include "dio.h"

DIO_DECLARE(fread, "fread stdio");

struct dio_fread_internal {
  FILE *dfile;
  char *buf;
};

DIO *dio_fread_open(char *path)
{
  struct dio_fread_internal *freadi;
  FILE *rfile;

  if ((rfile = fopen(path, "r")) == NULL)
    return NULL;

  freadi = g_new0(struct dio_fread_internal, 1);
  freadi->dfile = rfile;

  return freadi;
}

char *dio_fread_data(DIO *data)
{
  struct stat statbuf;
  struct dio_fread_internal *freadi = (struct dio_fread_internal *)data;

  if (freadi->buf)
    return freadi->buf;

  if (fstat(fileno(freadi->dfile), &statbuf) < 0)
    return NULL;

  freadi->buf = g_new0(char, statbuf.st_size + 1);
  fread(freadi->buf, statbuf.st_size, sizeof(char), freadi->dfile);
  return freadi->buf;
}

void dio_fread_close(DIO *data)
{
  struct dio_fread_internal *freadi = (struct dio_fread_internal *)data;

  fclose(freadi->dfile);
  if (freadi->buf)
    g_free(freadi->buf);
  g_free(freadi);
}

