/*
 * mmap() file io.
 */
#include <sys/mman.h>
#include "fuzzy.h"
#include "dio.h"

DIO_DECLARE(mmap, "mmap syscall");

struct dio_mmap_internal {
  int fd;
  char *data;
  int length;
};

DIO *dio_mmap_open(char *path)
{
  struct dio_mmap_internal *mmapi;
  struct stat statbuf;
  int fd, length;
  void *map;

  if (!path) {
    dlog("no path");
    return NULL;
  }
  if ((fd = open(path, O_RDONLY)) < 0) {
    dlog("open fail");
    return NULL;
  }
  if (fstat(fd, &statbuf) < 0) {
    dlog("fstat fail");
    return NULL;
  }

  length = statbuf.st_size;

  if ((map = mmap(0, length, PROT_READ | PROT_WRITE, MAP_FILE | MAP_PRIVATE, fd, 0)) == MAP_FAILED) {
    dlog("mmap fail");
    return NULL;
  }

  mmapi = g_new(struct dio_mmap_internal, 1);
  mmapi->fd = fd;
  mmapi->length = length;
  mmapi->data = map;
  return mmapi;
}

char *dio_mmap_data(DIO *data)
{
  struct dio_mmap_internal *mmapi = (struct dio_mmap_internal *)data;
  return mmapi->data;
}

void dio_mmap_close(DIO *data)
{
  struct dio_mmap_internal *mmapi = (struct dio_mmap_internal *)data;
  munmap(mmapi->data, mmapi->length);
  g_free(mmapi);
}
