/*
 * options.c - Unified parsing of line/file options.
 */
#include "fuzzy.h"
#include "dio.h"
#include "options.h"

char *parse_options(const char *input, struct options_list *data)
{
  int i;
  char *line, *key, *value, *string, *orig;

  g_return_val_if(!input || !*input, (char *)input);

  orig = string = g_strdup(input);

  while ((line = sread_line(&string))) {
    if (*line == '\0') {
      line = sread_line(&string);
      break;
    }
    if (*line == '*' || *line == '#' || (line[0] == '/' && line[1] == '/'))
      continue;
    if (!(value = strchr(line, ':')))
      break;

    key = line;
    *value++ = '\0';
    value = g_strchug(value);

    for (i = 0; data[i].key; i++)
      if (!g_strcasecmp(key, data[i].key)) {
        data[i].value = g_strdup(value);
        break;
      }

    if (!data[i].key)
      dlog("Unknown option: '%s'", key);
  }

  if (!line)
    dlog("No data left after options.");

  g_free(orig);
  return (char *)input + (line - orig);
}

void free_options(struct options_list *options)
{
  int i;

  for (i = 0; options[i].key; i++)
    g_free(options[i].value);
}
